#!/usr/bin/env python

import time

import praw

__all__ = ['PrawOAuth2Mini']

REDIRECT_URL = 'http://127.0.0.1:9999/authorize_callback'
SCOPES = ['identity']
EXPIRY_DURATION = 3500


class PrawOAuth2Mini:

    def __init__(self, reddit_client, app_key,
                 app_secret, access_token,
                 refresh_token='', scopes=SCOPES,
                 redirect_url=REDIRECT_URL):
        self.reddit_client = reddit_client
        self.app_key = app_key
        self.app_secret = app_secret
        self.access_token = access_token
        self.refresh_token = refresh_token
        self.scopes = set(scopes)
        self.redirect_url = redirect_url
        self.validity = 0

        self._set_app_info()
        self._set_access_credentials_first_time()

    def _set_validity(self):
        self.validity = time.time() + EXPIRY_DURATION

    def _is_token_expired(self):
        return time.time() > self.validity

    def _set_app_info(self):
        self.reddit_client.set_oauth_app_info(self.app_key, self.app_secret,
                                              self.redirect_url)

    def _set_access_credentials(self):
        self.reddit_client.set_access_credentials(self.scopes,
                                                  self.access_token,
                                                  self.refresh_token)
        self._set_validity()

    def _set_access_credentials_first_time(self):
        try:
            self._set_access_credentials()
        except praw.errors.OAuthInvalidToken:
            self.refesh()

    def _get_refresh_access(self):
        return self.reddit_client.refresh_access_information(
            self.refresh_token)

    def refesh(self):
        if self._is_token_expired():
            tokens = self._get_refresh_access()
            self.access_token = tokens['access_token']
            self.refresh_token = tokens['refresh_token']
            self._set_access_credentials()



#!/usr/bin/env python

from setuptools import setup, find_packages

try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
except (IOError, ImportError):
    long_description = open('README.md').read()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

version = '0.1.1'

setup(
    name='prawoauth2',
    version=version,
    install_requires=requirements,
    author='Avinash Sajjanshetty',
    author_email='hi@avi.im',
    packages=find_packages(),
    include_package_data=True,
    test_suite='tests',
    url='https://github.com/avinassh/prawoauth2/',
    license='MIT',
    description='Library to make your life easier using OAuth2 for PRAW',
    long_description=long_description,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ],
)



#!/usr/bin/env python

import webbrowser

import tornado.ioloop
import tornado.web

__all__ = ['PrawOAuth2Server']

REDIRECT_URL = 'http://127.0.0.1:65010/authorize_callback'
SCOPES = ['identity']
REFRESHABLE = True
CODE = None


class AuthorizationHandler(tornado.web.RequestHandler):

    def get(self):
        global CODE
        CODE = self.get_argument('code')
        self.write('successful (:')
        tornado.ioloop.IOLoop.current().stop()


class PrawOAuth2Server:

    def __init__(self, reddit_client, app_key, app_secret,
                 state, redirect_url=REDIRECT_URL, scopes=SCOPES,
                 refreshable=REFRESHABLE):
        self.reddit_client = reddit_client
        self.app_key = app_key
        self.app_secret = app_secret
        self.state = state
        self.redirect_url = redirect_url
        self.scopes = set(scopes)
        self.refreshable = refreshable
        self.code = None

        self._set_app_info()

    def _set_app_info(self):
        self.reddit_client.set_oauth_app_info(self.app_key, self.app_secret,
                                              self.redirect_url)

    def _get_auth_url(self):
        return self.reddit_client.get_authorize_url(self.state,
                                                    self.scopes,
                                                    self.refreshable)

    def start(self):
        global CODE
        url = self._get_auth_url()
        webbrowser.open(url)
        tornado.ioloop.IOLoop.current().start()
        self.code = CODE

    def get_access_codes(self):
        return self.reddit_client.get_access_information(self.code)

application = tornado.web.Application([
    (r'/authorize_callback', AuthorizationHandler),
])
application.listen(65010)
